/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2009 Drexel University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Joe Kopena <tjkopena@cs.drexel.edu>
 */

#include <errno.h>

#include <iostream>
#include <fstream>
#include <vector>
#include <sstream>

#include "net-debug.h"

#include "tokenizer.h"
#include "file-driver.h"

using namespace ns3;

bool
FileDriver::LoadLog(NetWorldModel &worldModel,
                    std::string filename)
{

  NET_STATE(filename);

  //  NetWorldTime time = interval;

  char str[BUFFER_LEN];
  std::vector<std::string> tokens;

  std::ifstream file(filename.c_str());
  if (file.is_open()) {

    //    NET_LOG("-- " << time << " --------------------------------");
    //    worldModel.NewSnapshot(start);

    uint32_t linecount = 1;

    while(!file.eof()) {

      file.getline(str, BUFFER_LEN);
      linecount++;

      Tokenizer::Tokenize(tokens, str, BUFFER_LEN);
      if (tokens.size() >= 1) {
        
        if (tokens[0] == "arena") {
          // NET_LOG("Found arena");

          std::istringstream ls(tokens[1]), ts(tokens[2]),
            rs(tokens[3]), bs(tokens[4]);
          double left, top, right, bottom;

          if (!(ls >> left) ||
              !(rs >> right) ||
              !(ts >> top) ||
              !(bs >> bottom)) {
            NET_ERR("Could not parse arena command, line " << linecount);
          } else {
            worldModel.SetArenaBounds(left, top, right, bottom);
          }

        } else {
          NetWorldTime t;

          int idx = tokens[0].find_first_not_of("0123456789.");
          std::string tval = tokens[0].substr(0, idx);
          std::string ttype = tokens[0].substr(idx, tokens[0].length());

          if (!(std::istringstream(tval) >> t)) {
            NET_ERR("Log parse error, no time or unknown command, line " <<
                    linecount);

          } else {

            if (ttype == "s") {
              t *= (NetWorldTime) 1000;
            } else if (ttype == "ns") {
              t /= 1000000;
            }

            if (tokens[1] == "position") {
              // NET_LOG("Found position at t " << t);
              //            worldModel.PushEvent(new NetWorldEvent(t));

              uint32_t id;
              double x, y, z, xv, yv, zv;

              if (tokens.size() < 9) {
                NET_ERR("Log parse error, position command with not enough " <<
                        " parameters, line " <<
                        linecount);
              } else if (!(std::istringstream(tokens[2]) >> id)) {
                NET_ERR("Parse error, position command with no id, line " <<
                        linecount);
              } else if (!(std::istringstream(tokens[3]) >> x)) {
                NET_ERR("Parse error, position command with no x, line " <<
                        linecount);
              } else if (!(std::istringstream(tokens[4]) >> y)) {
                NET_ERR("Parse error, position command with no y, line " <<
                        linecount);
              } else if (!(std::istringstream(tokens[5]) >> z)) {
                NET_ERR("Parse error, position command with no z, line " <<
                        linecount);
              } else if (!(std::istringstream(tokens[6]) >> xv)) {
                NET_ERR("Parse error, position command with no xv, line " <<
                        linecount);
              } else if (!(std::istringstream(tokens[7]) >> yv)) {
                NET_ERR("Parse error, position command with no yv, line " <<
                        linecount);
              } else if (!(std::istringstream(tokens[8]) >> zv)) {
                NET_ERR("Parse error, position command with no zv, line " <<
                        linecount);
              } else {
                worldModel.PushEvent(new NetWorldPositionEvent(t, id,
                                                               x, y, z,
                                                               xv, yv, zv));
              }

            } else if (tokens[1] == "ip") {
              // NET_LOG("Found ip");

              std::istringstream idstr(tokens[2]);
              uint32_t id;
              if (tokens.size() < 4) {
                NET_ERR("Parse error, ip command with not enough " <<
                        " parameters, line " <<
                        linecount);
              } else if (!(idstr >> id)) {
                NET_ERR("Parse error, ip command with no id, line " <<
                        linecount);
              } else {
                worldModel.PushEvent(new NetWorldIPEvent(t, id, tokens[3]));
              }

              // end ip command
            } else if (tokens[1] == "link") {
              // NET_LOG("Found link");

              std::istringstream idstr(tokens[2]);
              uint32_t id;
              if (tokens.size() < 5) {
                NET_ERR("Parse error, link command with not enough " <<
                        " parameters, line " <<
                        linecount);
              } else if (!(idstr >> id)) {
                NET_ERR("Parse error, link command with no id, line " <<
                        linecount);
              } else if (tokens[3] == "") {
                NET_ERR("Parse error, link command with no target, line " <<
                        linecount);
              } else if (tokens[4] == "") {
                NET_ERR("Parse error, link command with no style, line " <<
                        linecount);
              } else if (!(tokens[4].substr(0,1) == "+" ||
                           tokens[4].substr(0,1) == "-")) {
                NET_ERR("Parse error, link command unknown style command, " <<
                        "not +/-; line " << linecount << ": " <<
                        tokens[4].substr(0,1));
              } else {
                worldModel.PushEvent
                  (new NetWorldLinkStateEvent(t, id, tokens[3],
                                              (tokens[4].substr(0,1)=="+"),
                                              tokens[4].substr
                                              (1, tokens[4].length())));
              }
              // end link command
            } else if (tokens[1] == "state") {

              std::istringstream idstr(tokens[2]);
              uint32_t id;
              if (tokens.size() < 4) {
                NET_ERR("Parse error, state command with not enough " <<
                        " parameters, line " <<
                        linecount);
              } else if (!(idstr >> id)) {
                NET_ERR("Parse error, state command with no id, line " <<
                        linecount);
              } else if (tokens[3] == "") {
                NET_ERR("Parse error, state command with no state, line " <<
                        linecount);
              } else if (!(tokens[3].substr(0,1) == "+" ||
                           tokens[3].substr(0,1) == "-")) {
                NET_ERR("Parse error, state command unknown style command, " <<
                        "not +/-; line " << linecount << ": " <<
                        tokens[3].substr(0,1));
              } else {
                worldModel.PushEvent
                  (new NetWorldNodeStateEvent(t, id,
                                              (tokens[3].substr(0,1)=="+"),
                                              tokens[3].substr
                                              (1, tokens[3].length())));
              }

              // end state command
            } else {
              NET_ERR("Unknown command " << tokens[1] <<
                      "; line " << linecount);
            }

            // end time value was found
          }

          // end else was not an arena command
        }

        // end parse some tokens...
      }
      tokens.clear();

      // end looping over file
    }
    file.close();
  } else {
    NET_ERR("Could not open log file " << filename << " for read.");
    return true;
  }

  return false;

  // end FileDriver::LoadLog
}
