/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2009 Drexel University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Joe Kopena <tjkopena@cs.drexel.edu>
 */

#include "net-debug.h"

#include <gtkmm/scrolledwindow.h>
#include <gtkmm/main.h>
#include <gtkmm/window.h>

#include "net-view-area.h"

#include "net-world-model.h"
#include "file-driver.h"
#include "animation-model.h"
#include "time-slider.h"
#include "link-style-driver.h"
#include "node-style-driver.h"
#include "net-world-time.h"

int
main(int argc, char** argv)
{
  std::string filename("");
  std::string linkStyleFile("");
  std::string nodeStyleFile("");

  bool hasInterval = false;
  ns3::NetWorldTime interval = 100;

  //-- Setup GTK
  Gtk::Main kit(argc, argv);

  NET_STATE("Running " << argv[0]);

  if (argc <= 1) {
    NET_ERR("Must specify log file to run.");
    return -1;
  }

  int i = 1;
  while (i < argc) {

    if (!strncmp(argv[i], "-linkstyles", 11)) {
      if (argc-1 <= i+1) {
        NET_ERR("Link styles command, but no filename.");
        return -1;
      }

      i++;
      linkStyleFile = argv[i];
      NET_LOG("Link style command " << linkStyleFile);

      // end linkstyles command
    } else if (!strncmp(argv[i], "-nodestyles", 11)) {
      if (argc-1 <= i+1) {
        NET_ERR("Node styles command, but no filename.");
        return -1;
      }

      i++;
      nodeStyleFile = argv[i];
      NET_LOG("Node style command " << nodeStyleFile);

      // end nodestyles command
    } else if (!strncmp(argv[i], "-interval", 9)) {
      if (argc-1 <= i+1) {
        NET_ERR("Node styles command, but no filename.");
        return -1;
      }

      i++;
      if (!(std::istringstream(argv[i]) >> interval)) {
        NET_ERR("Interval command parameter could not be parsed.");
      } else {
        hasInterval = true;
        NET_LOG("Interval command " << interval);
      }

      // end interval command
    }

    i++;
    // end looping over args
  }

  filename = argv[argc-1];
  NET_LOG("Log file " << filename);

  Gtk::Window win;
  ns3::NetViewArea netView;

  win.set_title("ns-3 Decorator");
  win.set_border_width(0);
  win.set_size_request(410, 440);

  Gtk::ScrolledWindow scrolledWindow;
  scrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

  //-- Create the 2d network viewer and place it in scrollbars
  scrolledWindow.add(netView);

  ns3::TimeSlider playback;

  Gtk::VBox box;
  win.add(box);
  box.pack_start(scrolledWindow);

  Gtk::HBox hbox;
  box.pack_start(hbox, Gtk::PACK_SHRINK);

  //  Gtk::Button pButton(Gtk::Stock::MEDIA_STOP);
  //  hbox.pack_start(pButton, Gtk::PACK_SHRINK);
  hbox.pack_start(playback, Gtk::PACK_EXPAND_WIDGET);


  scrolledWindow.show();
  playback.show();
  //  pButton.show();
  hbox.show();
  box.show();

  win.show_all_children();

  //-- Link style loading --------------------------------------
  // Setup the style libraries
  ns3::LinkStyleMap linkStyles;

  // Setup some default styles
  linkStyles["weak"] = ns3::LinkStyle(0, 1, 0, 1);
  linkStyles["strong"] = ns3::LinkStyle(10);
  linkStyles["fancy"] = ns3::LinkStyle(4, 1, 0, 0.5, 1);

  if (linkStyleFile != "") {
    // Potentially override the default styles
    ns3::LinkStyleDriver::LoadLinkStyles(linkStyles, linkStyleFile);

    for (ns3::LinkStyleMap::iterator i = linkStyles.begin();
         i != linkStyles.end(); i++) {
      NET_LOG("Found style " << ((*i).first));
    }

    // end parsing link style file
  }


  //-- Node style loading --------------------------------------

  ns3::NodeStyleMap nodeStyles;

  // Setup some default styles
  nodeStyles["command"] = ns3::NodeStyle(0.25, 0, 0, 1, 1);
  nodeStyles["user"] = ns3::NodeStyle(4);
  nodeStyles["client"] = ns3::NodeStyle(0, 1, 0, 1);

  if (nodeStyleFile != "") {
    // Potentially override the default styles
    ns3::NodeStyleDriver::LoadNodeStyles(nodeStyles, nodeStyleFile);

    for (ns3::NodeStyleMap::iterator i = nodeStyles.begin();
         i != nodeStyles.end(); i++) {
      NET_LOG("Found style " << ((*i).first));
    }

    // end parsing node style file
  }

  netView.SetLinkStyleMap(linkStyles);
  netView.SetNodeStyleMap(nodeStyles);

  //-- Create the network model
  ns3::NetWorldModel worldModel;

  // Create the log loading file driver
  ns3::FileDriver::LoadLog(worldModel, filename);

  // Connect the world model to the GUI
  playback.SetWorldModel(&worldModel);
  netView.SetWorldModel(&worldModel);

  ns3::AnimationModel anim;
  anim.SetWorldModel(&worldModel);
  if (hasInterval) {
    anim.SetInterval(interval);
  }
  anim.AddNotifier(&netView);
  anim.AddNotifier(&playback);

  anim.ResetTo(0);
  anim.Start();

  Gtk::Main::run(win);

  /*
  ns3::NetWorldTime lastT = worldModel.GetLastTimestamp();
  ns3::NetWorldTime interval = 0;
  while (!worldModel.End()) {
    worldModel.ProcessUpTo(interval);
    interval += lastT/10;
  }
  NET_STATE("Last event: " << lastT);
  */

  return 0;

  // end main
}
